from requests import Response
from neuronpedia.requests.base_request import (
    NPRequest,
)
from neuronpedia.np_vector import NPVector

ChatMessage = dict[str, str]


class SteerChatRequest(NPRequest):
    def __init__(
        self,
    ):
        super().__init__("steer-chat")

    def steer(
        self,
        model_id: str,
        vectors: list[NPVector],
        default_chat_messages: list[ChatMessage] = [{"role": "user", "content": "Write a one sentence story."}],
        steered_chat_messages: list[ChatMessage] = [{"role": "user", "content": "Write a one sentence story."}],
        temperature: float = 0.5,
        n_tokens: int = 32,
        freq_penalty: float = 2,
        seed: int = 16,
        strength_multiplier: float = 4,
        steer_special_tokens: bool = True,
    ) -> Response:
        # convert the vectors to the feature format
        features = [
            {
                "modelId": vector.model_id,
                "layer": vector.source,
                "index": vector.index,
                "strength": vector.default_steer_strength,
            }
            for vector in vectors
        ]
        payload = {
            "modelId": model_id,
            "features": features,
            "defaultChatMessages": default_chat_messages,
            "steeredChatMessages": steered_chat_messages,
            "temperature": temperature,
            "n_tokens": n_tokens,
            "freq_penalty": freq_penalty,
            "seed": seed,
            "strength_multiplier": strength_multiplier,
            "steer_special_tokens": steer_special_tokens,
        }
        return self.send_request(method="POST", json=payload)



from dataclasses import dataclass
import os
from typing import List


@dataclass
class NPVector:
    """A Vector returned by the Neuronpedia API."""

    label: str
    model_id: str
    source: str
    index: str
    values: List[float]
    hook_name: str
    default_steer_strength: float | None
    url: str | None = None

    def __post_init__(self):
        if not self.url:
            USE_LOCALHOST = os.getenv("USE_LOCALHOST", False)
            BASE_URL = "https://neuronpedia.org/api" if not USE_LOCALHOST else "http://localhost:3000/api"
            self.url = f"{BASE_URL}/{self.model_id}/{self.source}/{self.index}"

    def __eq__(self, other: "NPVector") -> bool:
        return (
            self.model_id == other.model_id
            and self.source == other.source
            and self.index == other.index
            and self.label == other.label
            and self.hook_name == other.hook_name
            and self.values == other.values
            and self.default_steer_strength == other.default_steer_strength
        )

    def delete(self):
        # import here to avoid circular import
        from neuronpedia.requests.vector_request import VectorRequest

        return VectorRequest().delete(self)

    def steer_chat(self, steered_chat_messages: list[dict[str, str]]):
        # import here to avoid circular import
        from neuronpedia.requests.steer_request import SteerChatRequest

        return SteerChatRequest().steer(
            model_id=self.model_id, vectors=[self], steered_chat_messages=steered_chat_messages
        )

    @classmethod
    def get(cls, model_id: str, source: str, index: str) -> "NPVector":
        # import here to avoid circular import
        from neuronpedia.requests.vector_request import VectorRequest

        return VectorRequest().get(model_id, source, index)

    @classmethod
    def new(
        cls,
        label: str,
        model_id: str,
        layer_num: int,
        hook_type: str,
        vector: list[float],
        default_steer_strength: float | None = 10,
    ) -> "NPVector":
        # import here to avoid circular import
        from neuronpedia.requests.vector_request import VectorRequest

        np_vector = VectorRequest().new(
            label=label,
            model_id=model_id,
            layer_num=layer_num,
            hook_type=hook_type,
            vector=vector,
            default_steer_strength=default_steer_strength,
        )

        return np_vector



import json
import os
import requests
from dotenv import load_dotenv

load_dotenv()


class NPKeyMissingError(Exception):
    """The environment variable 'NEURONPEDIA_API_KEY' is not set."""

    pass


class NPUnauthorizedError(Exception):
    """Unauthorized. Please check your NEURONPEDIA_API_KEY environment variable."""

    pass


class NPRateLimitError(Exception):
    """Rate limit exceeded. Please try again later or email support@neuronpedia.org to raise your limit."""

    pass


class NPInvalidResponseError(Exception):
    """The API returned an invalid JSON response. Please contact support@neuronpedia.org."""

    pass


class NPRequest:
    USE_LOCALHOST = os.getenv("USE_LOCALHOST", False)
    BASE_URL = "https://neuronpedia.org/api" if not USE_LOCALHOST else "http://localhost:3000/api"

    def __init__(
        self,
        endpoint: str,
    ):
        if not self.BASE_URL:
            raise ValueError("BASE_URL must be set in a subclass or globally.")
        self.endpoint = endpoint
        self.api_key = self._get_api_key()

    @staticmethod
    def _get_api_key():
        """Retrieve and validate the NEURONPEDIA_API_KEY environment variable."""
        api_key = os.getenv("NEURONPEDIA_API_KEY")
        if not api_key:
            raise NPKeyMissingError("The environment variable 'NEURONPEDIA_API_KEY' is not set.")
        return api_key

    def get_url(
        self,
    ):
        """Construct the full URL for the request."""
        return f"{self.BASE_URL}/{self.endpoint}"

    def send_request(
        self,
        uri: str = "",
        method: str = "POST",
        **kwargs,
    ):
        """Send an HTTP request."""
        headers = kwargs.get(
            "headers",
            {},
        )
        headers["X-Api-Key"] = self.api_key
        url = f"{self.BASE_URL}/{self.endpoint}/{uri}"
        print(f"Sending {method} request to {url}")
        print(f"Body: {json.dumps(kwargs.get('json', {}), indent=2)}")

        response = requests.request(
            method,
            url,
            headers=headers,
            **kwargs,
        )

        if response.status_code == 401:
            raise NPUnauthorizedError("Unauthorized. Please check your NEURONPEDIA_API_KEY environment variable.")
        elif response.status_code == 404:
            raise requests.exceptions.HTTPError(f"Resource not found: {response.status_code}")
        elif response.status_code == 429:
            raise NPRateLimitError(
                "You've exceeded the API rate limit. Try later or email support@neuronpedia.org to raise your limit."
            )
        elif 400 <= response.status_code < 500:
            raise requests.exceptions.HTTPError(f"Request failed with status {response.status_code}: {response.json()}")
        elif 500 <= response.status_code < 600:
            raise requests.exceptions.HTTPError(
                f"Server error occurred: {response.status_code}. Try again later or contact support@neuronpedia.org."
            )
        response.raise_for_status()  # Raise an exception for HTTP errors
        try:
            response_json = response.json()
            print("Response:")
            print(json.dumps(response_json, indent=2))
            return response_json
        except requests.exceptions.JSONDecodeError:
            print(f"Error: Response text: {response.text[:1024]}")
            raise NPInvalidResponseError(
                "The API returned an invalid JSON response. Please contact support@neuronpedia.org."
            )
